


# Copyright 2016 Joshua R. Davis
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not 
# use this file except in compliance with the License. You may obtain a copy 
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software 
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT 
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the 
# License for the specific language governing permissions and limitations 
# under the License.



### INTRODUCTION ###

# Anisotropy of magnetic susceptibility (AMS) is a technique for measuring the 
# magnetic fabric of a rock sample. X-ray computed tomography (XRCT) is a 
# technique for imaging individual clasts inside a sample based on X-ray 
# opacity contrast. Given a rock sample, it is natural to ask whether the AMS 
# ellipsoid is simply the average of the XRCT ellipsoids. Or maybe it is the 
# average of the highly magnetic XRCT ellipsoids? That would be nice to know, 
# because AMS is easier/cheaper/commoner than XRCT. To some extent these 
# questions can be answered using inference on ellipsoids.



### PROBLEM ###

# Load one AMS ellipsoid from a file (Garibaldi et al., in prep). The AGICO 
# software normalizes the ellipsoids in a strange (to me) way. So we de-
# normalize them and re-normalize them. The meanSuscepts argument is needed 
# for the de-normalization. You get it from a separate AGICO output file. When 
# there are multiple ellipsoids in the file, there should be one mean 
# susceptibility for each ellipsoid.
ams205 <- geoEllipsoidDataFromAGICOFile("data/HU150205ams.tsv", meanSuscepts=c(8.957E-03), doNormalize=TRUE)
ams205 <- listFromDataFrame(ams205)[[1]]

# Load 4,001 XRCT ellipsoids from the same sample. By the way, all of these 
# data are from the Huemul plutonic complex, Chile.
xrct205 <- geoEllipsoidDataFromAvizoFile("data/HU150205mafic.tsv", doNormalize=TRUE)

# Inspect the shapes, with AMS superimposed in red. The AMS ellipsoid is much 
# closer to spherical than the XRCT ellipsoids. Is that consistent with the 
# AMS being the mean of the XRCT?
ellHsuNadaiPlot(c(xrct205$logA, list(ams205$logA)),
                colors=c(replicate(length(xrct205$logA), "black"), "red"))
quantile(sapply(xrct205$logA, ellOctahedralShearStrain))

# Inspect the orientations. They look fairly uniform, although there is a 
# little clumping. You can detect the clumping with Kamb contouring if you 
# want, but it takes a few minutes to get a good image.
ellEqualVolumePlot(xrct205$rotation, xrct205$logA, simplePoints=TRUE)
ellEqualAreaPlot(xrct205$rotation, xrct205$logA)
#lineKambPlot(lapply(xrct205$rotation, function(r) r[1,]), numNonAdapt=5)
#lineKambPlot(lapply(xrct205$rotation, function(r) r[2,]), numNonAdapt=5)
#lineKambPlot(lapply(xrct205$rotation, function(r) r[3,]), numNonAdapt=5)

# Inspect the ellipsoid vectors, with the AMS superimposed in red. It seems 
# plausible that the AMS ellipsoid could be the mean of the XRCT ellipsoids. 
# But keep in mind that regions of high density might not be recognizable in 
# this plot.
ellPairsPlot(c(xrct205$vector, list(ams205$vector)),
             colors=c(replicate(length(xrct205$vector), "black"), "red"))

# So we are left wondering whether AMS could be the mean of XRCT. If that were 
# true in general, then the two kinds of data would be interchangeable to some 
# extent --- for example, we could use AMS on rocks where XRCT gives poor 
# results. If it were generally false that AMS is the mean of XRCT, then the 
# difference between them might merit further investigation.

# We can address this question using inference. It is worth noting that all of 
# our inference methods assume that the data (in this case, the XRCT) are 
# 'independent and identically distributed' (IID). That is, they all arise 
# from the same underlying population, and they are a 'random' sample from 
# that population (as opposed to a systematically biased sample, for example).



### HYPOTHESIS TESTS ###

# Now we test whether the AMS ellipsoid is the mean of the XRCT ellipsoids 
# using two methods. The first is a standard asymptotic multivariate statistics 
# test provided by the R package ICSNP. The second is my own custom bootstrap 
# method, based on percentiles of Mahalanobis distance.
xrctInfT2 <- ellHotellingT2Inference(xrct205$vector, ams205$vector)
xrctInf <- ellBootstrapInference(xrct205$vector, numBoots=1000)

# Here are the p-values for the null hypothesis that the difference is zero. 
# In plain English, what do they mean about these XRCT and AMS data?
xrctInfT2$p.value
xrctInf$pvalue(ams205$vector)



### CONFIDENCE REGION ###

# The S-type bootstrap inference can be processed in exactly the same way. Its 
# results are nearly identical. Let's skip it.

# The bootstrapping method produces a 5D confidence ellipsoid.
xrctElls <- lapply(ellHighEllipsoidVectors(xrctInf$covarInv, xrctInf$center, xrctInf$q095^2), 
                   ellEllipsoidFromVector)
ellPairsPlot(c(lapply(xrctElls, function(ell) ell$vector), list(ams205$vector)),
             color=c(replicate(length(xrctElls), "black"), "red"))
ellHsuNadaiPlot(c(lapply(xrctElls, function(ell) ell$logA), list(ams205$logA)),
                color=c(replicate(length(xrctElls), "black"), "red"))
ellEqualVolumePlot(c(lapply(xrctElls, function(ell) ell$rotation), list(ams205$rotation)),
                   c(lapply(xrctElls, function(ell) ell$logA), list(ams205$logA)),
                   color=c(replicate(length(xrctElls), "white"), "red"), simplePoints=TRUE)
ellEqualAreaPlot(c(lapply(xrctElls, function(ell) ell$rotation), list(ams205$rotation)),
                 c(lapply(xrctElls, function(ell) ell$logA), list(ams205$logA)),
                 color=c(replicate(length(xrctElls), "black"), "red"))



### CONCLUSION ###

# Even though the plots make AMS look like the mean of XRCT, there are several 
# statistical tests that show it isn't (for this rock sample).


